package dao

import (
	"enterprise/common/model"
	"gorm.io/gorm"
	"time"
)

type UserConfigDao struct {
}

func NewUserConfigDao() *UserConfigDao {
	return &UserConfigDao{}
}

func (d *UserConfigDao) TableName() string {
	return "user_config"
}

func (d *UserConfigDao) Create(o *model.UserConfig) (int64, error) {
	o.CreateTime = time.Now().Unix()
	res := GetDB().Table(d.TableName()).Create(o)
	return o.Id, res.Error
}

func (d *UserConfigDao) Update(o *model.UserConfig) error {
	o.UpdateTime = time.Now().Unix()
	tx := GetDB().Table(d.TableName())
	res := tx.Save(o)
	return res.Error
}

func (d *UserConfigDao) Delete(id int64) error {
	res := GetDB().Table(d.TableName()).Delete(&model.UserConfig{}, id)
	return res.Error
}

func (d *UserConfigDao) Get(id int64) (*model.UserConfig, error) {
	var u model.UserConfig
	tx := GetDB().Table(d.TableName())
	tx = tx.Where("id = ?", id)
	res := tx.First(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return &u, nil
}

func (d *UserConfigDao) GetByUsername(username string) (*model.UserConfig, error) {
	var u model.UserConfig
	tx := GetDB().Table(d.TableName())
	tx = tx.Where("username = ?", username)
	res := tx.First(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return &u, nil
}

func (d *UserConfigDao) Query(status int) ([]*model.UserConfig, error) {
	var u []*model.UserConfig
	tx := GetDB().Table(d.TableName())

	if status != 0 {
		tx = tx.Where("status = ?", status)
	}

	res := tx.Find(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return u, nil
}
